@@ -2,7 +2,7 @@
  * Asterisk -- An open source telephony toolkit.
  *
  * A full-featured Find-Me/Follow-Me Application
- * 
+ *
  * Copyright (C) 2005-2006, BJ Weschke All Rights Reserved.
  *
  * BJ Weschke <bweschke@btwtech.com>
@@ -30,7 +30,7 @@
  * \addtogroup configuration_file Configuration Files
  */
 
-/*! 
+/*!
  * \page followme.conf followme.conf
  * \verbinclude followme.conf.sample
  */
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <signal.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_SPOOL_DIR */
@@ -178,6 +176,7 @@ struct call_followme {
 	char plsholdprompt[PATH_MAX];	/*!< Sound prompt name and path */
 	char statusprompt[PATH_MAX];	/*!< Sound prompt name and path */
 	char sorryprompt[PATH_MAX];	/*!< Sound prompt name and path */
+	char connprompt[PATH_MAX];	/*!< Sound prompt name and path */
 
 	AST_LIST_HEAD_NOLOCK(numbers, number) numbers;	   /*!< Head of the list of follow-me numbers */
 	AST_LIST_HEAD_NOLOCK(blnumbers, number) blnumbers; /*!< Head of the list of black-listed numbers */
@@ -214,6 +213,7 @@ struct fm_args {
 	char plsholdprompt[PATH_MAX];	/*!< Sound prompt name and path */
 	char statusprompt[PATH_MAX];	/*!< Sound prompt name and path */
 	char sorryprompt[PATH_MAX];	/*!< Sound prompt name and path */
+	char connprompt[PATH_MAX];	/*!< Sound prompt name and path */
 	struct ast_flags followmeflags;
 };
 
@@ -279,6 +279,7 @@ static char optionsprompt[PATH_MAX] = "followme/options";
 static char plsholdprompt[PATH_MAX] = "followme/pls-hold-while-try";
 static char statusprompt[PATH_MAX] = "followme/status";
 static char sorryprompt[PATH_MAX] = "followme/sorry";
+static char connprompt[PATH_MAX] = "";
 
 
 static AST_RWLIST_HEAD_STATIC(followmes, call_followme);
@@ -335,20 +336,21 @@ static void init_profile(struct call_followme *f, int activate)
 	ast_copy_string(f->plsholdprompt, plsholdprompt, sizeof(f->plsholdprompt));
 	ast_copy_string(f->statusprompt, statusprompt, sizeof(f->statusprompt));
 	ast_copy_string(f->sorryprompt, sorryprompt, sizeof(f->sorryprompt));
+	ast_copy_string(f->connprompt, connprompt, sizeof(f->connprompt));
 	if (activate) {
 		f->active = 1;
 	}
 }
 
-   
-   
+
+
 /*! \brief Set parameter in profile from configuration file */
 static void profile_set_param(struct call_followme *f, const char *param, const char *val, int linenum, int failunknown)
 {
 
-	if (!strcasecmp(param, "musicclass") || !strcasecmp(param, "musiconhold") || !strcasecmp(param, "music")) 
+	if (!strcasecmp(param, "musicclass") || !strcasecmp(param, "musiconhold") || !strcasecmp(param, "music"))
 		ast_copy_string(f->moh, val, sizeof(f->moh));
-	else if (!strcasecmp(param, "context")) 
+	else if (!strcasecmp(param, "context"))
 		ast_copy_string(f->context, val, sizeof(f->context));
 	else if (!strcasecmp(param, "enable_callee_prompt"))
 		f->enable_callee_prompt = ast_true(val);
@@ -358,17 +360,19 @@ static void profile_set_param(struct call_followme *f, const char *param, const
 		ast_copy_string(f->nextindp, val, sizeof(f->nextindp));
 	else if (!strcasecmp(param, "call-from-prompt") || !strcasecmp(param, "call_from_prompt"))
 		ast_copy_string(f->callfromprompt, val, sizeof(f->callfromprompt));
-	else if (!strcasecmp(param, "followme-norecording-prompt") || !strcasecmp(param, "norecording_prompt")) 
+	else if (!strcasecmp(param, "followme-norecording-prompt") || !strcasecmp(param, "norecording_prompt"))
 		ast_copy_string(f->norecordingprompt, val, sizeof(f->norecordingprompt));
-	else if (!strcasecmp(param, "followme-options-prompt") || !strcasecmp(param, "options_prompt")) 
+	else if (!strcasecmp(param, "followme-options-prompt") || !strcasecmp(param, "options_prompt"))
 		ast_copy_string(f->optionsprompt, val, sizeof(f->optionsprompt));
 	else if (!strcasecmp(param, "followme-pls-hold-prompt") || !strcasecmp(param, "pls_hold_prompt"))
 		ast_copy_string(f->plsholdprompt, val, sizeof(f->plsholdprompt));
-	else if (!strcasecmp(param, "followme-status-prompt") || !strcasecmp(param, "status_prompt")) 
+	else if (!strcasecmp(param, "followme-status-prompt") || !strcasecmp(param, "status_prompt"))
 		ast_copy_string(f->statusprompt, val, sizeof(f->statusprompt));
-	else if (!strcasecmp(param, "followme-sorry-prompt") || !strcasecmp(param, "sorry_prompt")) 
+	else if (!strcasecmp(param, "followme-sorry-prompt") || !strcasecmp(param, "sorry_prompt"))
 		ast_copy_string(f->sorryprompt, val, sizeof(f->sorryprompt));
-	else if (failunknown) {
+	else if (!strcasecmp(param, "followme-connecting-prompt") || !strcasecmp(param, "connecting_prompt")) {
+		ast_copy_string(f->connprompt, val, sizeof(f->connprompt));
+	} else if (failunknown) {
 		if (linenum >= 0)
 			ast_log(LOG_WARNING, "Unknown keyword in profile '%s': %s at line %d of followme.conf\n", f->name, param, linenum);
 		else
@@ -491,6 +495,13 @@ static int reload_followme(int reload)
 		ast_copy_string(sorryprompt, tmpstr, sizeof(sorryprompt));
 	}
 
+	if ((tmpstr = ast_variable_retrieve(cfg, "general", "connecting-prompt")) && !ast_strlen_zero(tmpstr)) {
+		ast_copy_string(connprompt, tmpstr, sizeof(connprompt));
+	} else if ((tmpstr = ast_variable_retrieve(cfg, "general", "connecting_prompt")) && !ast_strlen_zero(tmpstr)) {
+		ast_copy_string(connprompt, tmpstr, sizeof(connprompt));
+	}
+
+
 	/* Chug through config file */
 	while ((cat = ast_category_browse(cfg, cat))) {
 		int new = 0;
@@ -539,7 +550,7 @@ static int reload_followme(int reload)
 						numorder = atoi(tmp);
 						if (numorder < 0)
 							numorder = 0;
-					} else 
+					} else
 						numorder = 0;
 				} else {
 					timeout = 25;
@@ -548,7 +559,7 @@ static int reload_followme(int reload)
 
 				if (!numorder) {
 					idx = 1;
-					AST_LIST_TRAVERSE(&f->numbers, nm, entry) 
+					AST_LIST_TRAVERSE(&f->numbers, nm, entry)
 						idx++;
 					numorder = idx;
 				}
@@ -563,7 +574,7 @@ static int reload_followme(int reload)
 			var = var->next;
 		} /* End while(var) loop */
 
-		if (!new) 
+		if (!new)
 			ast_mutex_unlock(&f->lock);
 		else
 			AST_RWLIST_INSERT_HEAD(&followmes, f, entry);
@@ -601,7 +612,7 @@ static void clear_caller(struct findme_user *tmpuser)
 	tmpuser->ochan = NULL;
 }
 
-static void clear_unanswered_calls(struct findme_user_listptr *findme_user_list) 
+static void clear_unanswered_calls(struct findme_user_listptr *findme_user_list)
 {
 	struct findme_user *tmpuser;
 
@@ -643,7 +654,7 @@ static struct ast_channel *wait_for_winner(struct findme_user_listptr *findme_us
 	char *callfromname;
 	char *pressbuttonname;
 
-	/* ------------ wait_for_winner_channel start --------------- */ 
+	/* ------------ wait_for_winner_channel start --------------- */
 
 	callfromname = ast_strdupa(tpargs->callfromprompt);
 	pressbuttonname = ast_strdupa(tpargs->optionsprompt);
@@ -652,7 +663,7 @@ static struct ast_channel *wait_for_winner(struct findme_user_listptr *findme_us
 
 	for (;;) {
 		to = 1000;
-		pos = 1; 
+		pos = 1;
 		livechannels = 0;
 		watchers[0] = caller;
 
@@ -804,7 +815,7 @@ static struct ast_channel *wait_for_winner(struct findme_user_listptr *findme_us
 						ast_channel_publish_dial(caller, winner, NULL, "ANSWER");
 						publish_dial_end_event(caller, findme_user_list, winner, "CANCEL");
 						tmpuser->answered = 1;
-						/* If call has been answered, then the eventual hangup is likely to be normal hangup */ 
+						/* If call has been answered, then the eventual hangup is likely to be normal hangup */
 						ast_channel_hangupcause_set(winner, AST_CAUSE_NORMAL_CLEARING);
 						ast_channel_hangupcause_set(caller, AST_CAUSE_NORMAL_CLEARING);
 						if (tpargs->enable_callee_prompt) {
@@ -954,7 +965,7 @@ static struct ast_channel *wait_for_winner(struct findme_user_listptr *findme_us
 							f->subclass.integer, ast_channel_name(winner));
 						break;
 					}
-				} 
+				}
 				if (!tpargs->enable_callee_prompt && tmpuser) {
 					ast_debug(1, "Taking call with no prompt\n");
 					ast_frfree(f);
@@ -1410,7 +1421,8 @@ static int app_exec(struct ast_channel *chan, const char *data)
 	ast_copy_string(targs->plsholdprompt, f->plsholdprompt, sizeof(targs->plsholdprompt));
 	ast_copy_string(targs->statusprompt, f->statusprompt, sizeof(targs->statusprompt));
 	ast_copy_string(targs->sorryprompt, f->sorryprompt, sizeof(targs->sorryprompt));
-	/* Copy the numbers we're going to use into another list in case the master list should get modified 
+	ast_copy_string(targs->connprompt, f->connprompt, sizeof(targs->connprompt));
+	/* Copy the numbers we're going to use into another list in case the master list should get modified
 	   (and locked) while we're trying to do a follow-me */
 	AST_LIST_HEAD_INIT_NOLOCK(&targs->cnumbers);
 	AST_LIST_TRAVERSE(&f->numbers, nm, entry) {
@@ -1497,6 +1509,14 @@ static int app_exec(struct ast_channel *chan, const char *data)
 		res = 0;
 	} else {
 		caller = chan;
+
+		/* Play "connecting" message to the winner, if configured. */
+		if (!ast_strlen_zero(targs->connprompt)) {
+			ast_autoservice_start(caller);
+			ast_stream_and_wait(outbound, targs->connprompt, "");
+			ast_autoservice_stop(caller);
+		}
+
 		/* Bridge the two channels. */
 
 		memset(&config, 0, sizeof(config));
@@ -1610,8 +1630,8 @@ static int unload_module(void)
  * Module loading including tests for configuration or dependencies.
  * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
  * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
- * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
- * configuration file or other non-critical problem return 
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the
+ * configuration file or other non-critical problem return
  * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
  */
 static int load_module(void)
