@@ -22,7 +22,7 @@
  * \author Anthony Minessale anthmct@yahoo.com
  *
  * \note Development of this app Sponsored/Funded by TAAN Softworks Corp
- * 
+ *
  * \ingroup applications
  */
 
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
