@@ -21,7 +21,7 @@
  * \brief Get ADSI CPE ID
  *
  * \author Mark Spencer <markster@digium.com>
- * 
+ *
  * \ingroup applications
  */
 
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -106,7 +104,7 @@ static int cpeid_exec(struct ast_channel *chan, const char *idata)
 					cpeid[0], cpeid[1], cpeid[2], cpeid[3]);
 			else
 				strcpy(data[1], "CPEID Unknown");
-			if (gotgeometry) 
+			if (gotgeometry)
 				snprintf(data[2], 80, "Geom: %dx%d, %d buttons", width, height, buttons);
 			else
 				strcpy(data[2], "Geometry unknown");
