@@ -21,17 +21,15 @@
  * \brief App to transmit an image
  *
  * \author Mark Spencer <markster@digium.com>
- * 
+ *
  * \ingroup applications
  */
 
 /*** MODULEINFO
 	<support_level>extended</support_level>
  ***/
- 
-#include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
+#include "asterisk.h"
 
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -92,7 +90,7 @@ static int sendimage_exec(struct ast_channel *chan, const char *data)
 	} else {
 		pbx_builtin_setvar_helper(chan, "SENDIMAGESTATUS", "FAILURE");
 	}
-	
+
 	return 0;
 }
 
@@ -107,4 +105,3 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Image Transmission Application");
-
