@@ -21,7 +21,7 @@
  * \brief App to set callerid presentation
  *
  * \author Mark Spencer <markster@digium.com>
- * 
+ *
  * \ingroup applications
  */
 
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -110,7 +108,7 @@ static int setcallerid_pres_exec(struct ast_channel *chan, const char *data)
 			(char *) data);
 		return 0;
 	}
-	
+
 	/* Set the combined caller id presentation. */
 	ast_channel_caller(chan)->id.name.presentation = pres;
 	ast_channel_caller(chan)->id.number.presentation = pres;
@@ -128,4 +126,3 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Set CallerID Presentation Application");
-
