@@ -26,13 +26,12 @@
  */
 
 /*** MODULEINFO
+	<use type="module">res_monitor</use>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
