@@ -1553,7 +1553,21 @@ UPDATE alembic_version SET version_num='4e2493ef32e6' WHERE alembic_version.vers
 
 /
 
--- Running upgrade 4e2493ef32e6 -> a6ef36f1309
+-- Running upgrade 4e2493ef32e6 -> 7f3e21abe318
+
+ALTER TABLE ps_endpoints ADD preferred_codec_only VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (preferred_codec_only IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='7f3e21abe318' WHERE alembic_version.version_num = '4e2493ef32e6'
+
+/
+
+-- Running upgrade 7f3e21abe318 -> a6ef36f1309
 
 ALTER TABLE ps_globals ADD ignore_uri_user_options VARCHAR(3 CHAR)
 
@@ -1563,7 +1577,7 @@ ALTER TABLE ps_globals ADD CONSTRAINT yesno_values CHECK (ignore_uri_user_option
 
 /
 
-UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '4e2493ef32e6'
+UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '7f3e21abe318'
 
 /
 
@@ -1765,7 +1779,21 @@ UPDATE alembic_version SET version_num='d7983954dd96' WHERE alembic_version.vers
 
 /
 
--- Running upgrade d7983954dd96 -> 164abbd708c
+-- Running upgrade d7983954dd96 -> 39959b9c2566
+
+ALTER TABLE ps_endpoints ADD max_audio_streams INTEGER
+
+/
+
+ALTER TABLE ps_endpoints ADD max_video_streams INTEGER
+
+/
+
+UPDATE alembic_version SET version_num='39959b9c2566' WHERE alembic_version.version_num = 'd7983954dd96'
+
+/
+
+-- Running upgrade 39959b9c2566 -> 164abbd708c
 
 ALTER TABLE ps_endpoints DROP CONSTRAINT pjsip_dtmf_mode_values_v2
 
@@ -1779,11 +1807,25 @@ ALTER TABLE ps_endpoints ADD CONSTRAINT pjsip_dtmf_mode_values_v3 CHECK (dtmf_mo
 
 /
 
-UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96'
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = '39959b9c2566'
 
 /
 
--- Running upgrade 164abbd708c -> f3d1c5d38b56
+-- Running upgrade 164abbd708c -> 44ccced114ce
+
+ALTER TABLE ps_endpoints ADD webrtc VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (webrtc IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='44ccced114ce' WHERE alembic_version.version_num = '164abbd708c'
+
+/
+
+-- Running upgrade 44ccced114ce -> f3d1c5d38b56
 
 ALTER TABLE ps_contacts ADD prune_on_boot VARCHAR(3 CHAR)
 
@@ -1793,7 +1835,7 @@ ALTER TABLE ps_contacts ADD CONSTRAINT yesno_values CHECK (prune_on_boot IN ('ye
 
 /
 
-UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c'
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '44ccced114ce'
 
 /
 
@@ -1821,3 +1863,69 @@ UPDATE alembic_version SET version_num='a1698e8bb9c5' WHERE alembic_version.vers
 
 /
 
+-- Running upgrade a1698e8bb9c5 -> 20abce6d1e3c
+
+ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR(13 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT pjsip_identify_by_values CHECK (identify_by IN ('username', 'auth_username', 'ip'))
+
+/
+
+UPDATE alembic_version SET version_num='20abce6d1e3c' WHERE alembic_version.version_num = 'a1698e8bb9c5'
+
+/
+
+-- Running upgrade 20abce6d1e3c -> de83fac997e2
+
+ALTER TABLE ps_endpoints ADD bundle VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (bundle IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='de83fac997e2' WHERE alembic_version.version_num = '20abce6d1e3c'
+
+/
+
+-- Running upgrade de83fac997e2 -> 041c0d3d1857
+
+ALTER TABLE ps_endpoints ADD dtls_auto_generate_cert VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (dtls_auto_generate_cert IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='041c0d3d1857' WHERE alembic_version.version_num = 'de83fac997e2'
+
+/
+
+-- Running upgrade 041c0d3d1857 -> 52798ad97bdf
+
+ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR2(80 CHAR)
+
+/
+
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '041c0d3d1857'
+
+/
+
+-- Running upgrade 52798ad97bdf -> d3e4284f8707
+
+ALTER TABLE ps_subscription_persistence ADD prune_on_boot VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_subscription_persistence ADD CONSTRAINT yesno_values CHECK (prune_on_boot IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='d3e4284f8707' WHERE alembic_version.version_num = '52798ad97bdf'
+
+/
+
