@@ -19,7 +19,7 @@
 /*! \file
  *
  * \brief JPEG File format support.
- * 
+ *
  * \arg File name extension: jpeg, jpg
  * \ingroup formats
  */
@@ -27,10 +27,8 @@
 /*** MODULEINFO
 	<support_level>extended</support_level>
  ***/
- 
-#include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
+#include "asterisk.h"
 
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
