@@ -23,14 +23,12 @@
  */
 
 /*** MODULEINFO
-	<use type="external">crypto</use>
+	<use type="module">res_crypto</use>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 #include "asterisk/app.h"
