@@ -69,7 +69,7 @@ enum ast_module_load_result {
 	AST_MODULE_LOAD_SUCCESS = 0,    /*!< Module loaded and configured */
 	AST_MODULE_LOAD_DECLINE = 1,    /*!< Module is not configured */
 	AST_MODULE_LOAD_SKIP = 2,       /*!< Module was skipped for some reason (For loader.c use only. Should never be returned by modules)*/
-	AST_MODULE_LOAD_PRIORITY = 3,   /*!< Module is not loaded yet, but is added to prioity heap */
+	AST_MODULE_LOAD_PRIORITY = 3,   /*!< Module is not loaded yet, but is added to priority list */
 	AST_MODULE_LOAD_FAILURE = -1,   /*!< Module could not be loaded properly */
 };
 
@@ -94,7 +94,21 @@ enum ast_module_support_level {
 	AST_MODULE_SUPPORT_DEPRECATED,
 };
 
-/*! 
+/*! Used to specify which modules should be returned by ast_module_helper. */
+enum ast_module_helper_type {
+	/*! Modules that are loaded by dlopen. */
+	AST_MODULE_HELPER_LOADED = 0,
+	/*! Running modules that include a reload callback. */
+	AST_MODULE_HELPER_RELOAD = 1,
+	/*! Modules that can be loaded or started. */
+	AST_MODULE_HELPER_LOAD,
+	/*! Modules that can be unloaded. */
+	AST_MODULE_HELPER_UNLOAD,
+	/*! Running modules */
+	AST_MODULE_HELPER_RUNNING,
+};
+
+/*!
  * \brief Load a module.
  * \param resource_name The name of the module to load.
  *
@@ -106,13 +120,13 @@ enum ast_module_support_level {
  */
 enum ast_module_load_result ast_load_resource(const char *resource_name);
 
-/*! 
+/*!
  * \brief Unload a module.
  * \param resource_name The name of the module to unload.
  * \param ast_module_unload_mode The force flag. This should be set using one of the AST_FORCE flags.
  *
  * This function unloads a module.  It will only unload modules that are not in
- * use (usecount not zero), unless #AST_FORCE_FIRM or #AST_FORCE_HARD is 
+ * use (usecount not zero), unless #AST_FORCE_FIRM or #AST_FORCE_HARD is
  * specified.  Setting #AST_FORCE_FIRM or #AST_FORCE_HARD will unload the
  * module regardless of consequences (NOT RECOMMENDED).
  *
@@ -135,7 +149,7 @@ int ast_unload_resource(const char *resource_name, enum ast_module_unload_mode);
  */
 enum ast_module_reload_result ast_module_reload(const char *name);
 
-/*! 
+/*!
  * \brief Notify when usecount has been changed.
  *
  * This function calulates use counts and notifies anyone trying to keep track
@@ -200,51 +214,50 @@ int ast_update_module_list_condition(int (*modentry)(const char *module, const c
 /*!
  * \brief Check if module with the name given is loaded
  * \param name Module name, like "chan_sip.so"
- * \retval 1 if true 
+ * \retval 1 if true
  * \retval 0 if false
  */
 int ast_module_check(const char *name);
 
-/*! 
+/*!
  * \brief Add a procedure to be run when modules have been updated.
  * \param updater The function to run when modules have been updated.
  *
  * This function adds the given function to a linked list of functions to be
- * run when the modules are updated. 
+ * run when the modules are updated.
  *
- * \retval 0 on success 
+ * \retval 0 on success
  * \retval -1 on failure.
  */
 int ast_loader_register(int (*updater)(void));
 
-/*! 
+/*!
  * \brief Remove a procedure to be run when modules are updated.
  * \param updater The updater function to unregister.
  *
  * This removes the given function from the updater list.
- * 
+ *
  * \retval 0 on success
  * \retval -1 on failure.
  */
 int ast_loader_unregister(int (*updater)(void));
 
-/*! 
+/*!
  * \brief Match modules names for the Asterisk cli.
  * \param line Unused by this function, but this should be the line we are
  *        matching.
- * \param word The partial name to match. 
+ * \param word The partial name to match.
  * \param pos The position the word we are completing is in.
  * \param state The possible match to return.
  * \param rpos The position we should be matching.  This should be the same as
  *        pos.
- * \param needsreload This should be 1 if we need to reload this module and 0
- *        otherwise.  This function will only return modules that are reloadble
- *        if this is 1.
+ * \param type The type of action that will be performed by CLI.
+ *             This argument accepts values of enum ast_module_helper_type.
  *
  * \retval A possible completion of the partial match.
  * \retval NULL if no matches were found.
  */
-char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, int needsreload);
+char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, int type);
 
 /* Opaque type for module handles generated by the loader */
 
@@ -303,12 +316,10 @@ struct ast_module_info {
 	enum ast_module_load_result (*load)(void);	/*!< register stuff etc. Optional. */
 	int (*reload)(void);			/*!< config etc. Optional. */
 	int (*unload)(void);			/*!< unload. called with the module locked */
-	int (*backup_globals)(void);		/*!< for embedded modules, backup global data */
-	void (*restore_globals)(void);		/*!< for embedded modules, restore global data */
 	const char *name;			/*!< name of the module for loader reference and CLI commands */
 	const char *description;		/*!< user friendly description of the module. */
 
-	/*! 
+	/*!
 	 * This holds the ASTERISK_GPL_KEY, signifiying that you agree to the terms of
 	 * the Asterisk license as stated in the ASTERISK_GPL_KEY.  Your module will not
 	 * load if it does not return the EXACT key string.
@@ -342,9 +353,9 @@ struct ast_module_user *__ast_module_user_add(struct ast_module *, struct ast_ch
 void __ast_module_user_remove(struct ast_module *, struct ast_module_user *);
 void __ast_module_user_hangup_all(struct ast_module *);
 
-#define ast_module_user_add(chan) __ast_module_user_add(ast_module_info->self, chan)
-#define ast_module_user_remove(user) __ast_module_user_remove(ast_module_info->self, user)
-#define ast_module_user_hangup_all() __ast_module_user_hangup_all(ast_module_info->self)
+#define ast_module_user_add(chan) __ast_module_user_add(AST_MODULE_SELF, chan)
+#define ast_module_user_remove(user) __ast_module_user_remove(AST_MODULE_SELF, user)
+#define ast_module_user_hangup_all() __ast_module_user_hangup_all(AST_MODULE_SELF)
 
 struct ast_module *__ast_module_ref(struct ast_module *mod, const char *file, int line, const char *func);
 void __ast_module_shutdown_ref(struct ast_module *mod, const char *file, int line, const char *func);
@@ -379,8 +390,6 @@ void __ast_module_unref(struct ast_module *mod, const char *file, int line, cons
 		load_func,                                                     \
 		reload_func,                                                   \
 		unload_func,                                                   \
-		NULL,                                                          \
-		NULL,                                                          \
 		AST_MODULE,                                                    \
 		desc,                                                          \
 		keystr,                                                        \
@@ -398,8 +407,13 @@ void __ast_module_unref(struct ast_module *mod, const char *file, int line, cons
 	{                                                                  \
 		ast_module_unregister(&__mod_info);                            \
 	}                                                                  \
+	struct ast_module *AST_MODULE_SELF_SYM(void)                       \
+	{                                                                  \
+		return __mod_info.self;                                        \
+	}                                                                  \
 	static const __attribute__((unused)) struct ast_module_info *ast_module_info = &__mod_info
 
+
 #define AST_MODULE_INFO_STANDARD(keystr, desc)              \
 	AST_MODULE_INFO(keystr, AST_MODFLAG_LOAD_ORDER, desc,   \
 			load_module,                                    \
@@ -431,72 +445,13 @@ void __ast_module_unref(struct ast_module *mod, const char *file, int line, cons
 /* forward declare this pointer in modules, so that macro/function
    calls that need it can get it, since it will actually be declared
    and populated at the end of the module's source file... */
+#if !defined(AST_IN_CORE)
 static const __attribute__((unused)) struct ast_module_info *ast_module_info;
-
-#if !defined(EMBEDDED_MODULE)
-#define __MODULE_INFO_SECTION
-#define __MODULE_INFO_GLOBALS
-#else
-/*
- * For embedded modules we need additional information to backup and
- * restore the global variables in the module itself, so we can unload
- * reload the module.
- * EMBEDDED_MODULE is defined as the module name, so the calls to make_var()
- * below will actually define different symbols for each module.
- */
-#define __MODULE_INFO_SECTION	__attribute__((section(".embed_module")))
-#define __MODULE_INFO_GLOBALS	.backup_globals = __backup_globals, .restore_globals = __restore_globals,
-
-#define make_var_sub(mod, type) __ ## mod ## _ ## type
-#define make_var(mod, type) make_var_sub(mod, type)
-
-extern void make_var(EMBEDDED_MODULE, bss_start);
-extern void make_var(EMBEDDED_MODULE, bss_end);
-extern void make_var(EMBEDDED_MODULE, data_start);
-extern void make_var(EMBEDDED_MODULE, data_end);
-
-static void * __attribute__((section(".embed_module"))) __global_backup;
-
-static int __backup_globals(void)
-{
-	size_t data_size = & make_var(EMBEDDED_MODULE, data_end) - & make_var(EMBEDDED_MODULE, data_start);
-
-	if (__global_backup)
-		return 0;
-
-	if (!data_size)
-		return 0;
-
-	if (!(__global_backup = ast_malloc(data_size)))
-		return -1;
-
-	memcpy(__global_backup, & make_var(EMBEDDED_MODULE, data_start), data_size);
-
-	return 0;
-}
-
-static void __restore_globals(void)
-{
-	size_t data_size = & make_var(EMBEDDED_MODULE, data_end) - & make_var(EMBEDDED_MODULE, data_start);
-	size_t bss_size = & make_var(EMBEDDED_MODULE, bss_end) - & make_var(EMBEDDED_MODULE, bss_start);
-
-	if (bss_size)
-		memset(& make_var(EMBEDDED_MODULE, bss_start), 0, bss_size);
-
-	if (!data_size || !__global_backup)
-		return;
-
-	memcpy(& make_var(EMBEDDED_MODULE, data_start), __global_backup, data_size);
-}
-#undef make_var
-#undef make_var_sub
-#endif /* EMBEDDED_MODULE */
+#endif
 
 #define AST_MODULE_INFO(keystr, flags_to_set, desc, fields...)	\
 	static struct ast_module_info 				\
-		__MODULE_INFO_SECTION				\
 		__mod_info = {					\
-		__MODULE_INFO_GLOBALS				\
 		.name = AST_MODULE,				\
 		.flags = flags_to_set,				\
 		.description = desc,				\
@@ -512,6 +467,10 @@ static void __restore_globals(void)
 	{ \
 		ast_module_unregister(&__mod_info); \
 	} \
+	struct ast_module *AST_MODULE_SELF_SYM(void)                       \
+	{                                                                  \
+		return __mod_info.self;                                        \
+	}                                                                  \
 	static const struct ast_module_info *ast_module_info = &__mod_info
 
 #define AST_MODULE_INFO_STANDARD(keystr, desc)              \
@@ -540,37 +499,37 @@ static void __restore_globals(void)
 
 #endif	/* plain C */
 
-/*! 
+/*!
  * \brief Register an application.
  *
  * \param app Short name of the application
  * \param execute a function callback to execute the application. It should return
  *                non-zero if the channel needs to be hung up.
  * \param synopsis a short description (one line synopsis) of the application
- * \param description long description with all of the details about the use of 
+ * \param description long description with all of the details about the use of
  *                    the application
- * 
- * This registers an application with Asterisk's internal application list. 
+ *
+ * This registers an application with Asterisk's internal application list.
  * \note The individual applications themselves are responsible for registering and unregistering
  *       and unregistering their own CLI commands.
- * 
- * \retval 0 success 
+ *
+ * \retval 0 success
  * \retval -1 failure.
  */
-#define ast_register_application(app, execute, synopsis, description) ast_register_application2(app, execute, synopsis, description, ast_module_info->self)
+#define ast_register_application(app, execute, synopsis, description) ast_register_application2(app, execute, synopsis, description, AST_MODULE_SELF)
 
-/*! 
+/*!
  * \brief Register an application using XML documentation.
  *
  * \param app Short name of the application
  * \param execute a function callback to execute the application. It should return
  *                non-zero if the channel needs to be hung up.
- * 
- * This registers an application with Asterisk's internal application list. 
+ *
+ * This registers an application with Asterisk's internal application list.
  * \note The individual applications themselves are responsible for registering and unregistering
  *       and unregistering their own CLI commands.
- * 
- * \retval 0 success 
+ *
+ * \retval 0 success
  * \retval -1 failure.
  */
 #define ast_register_application_xml(app, execute) ast_register_application(app, execute, NULL, NULL)
@@ -597,14 +556,14 @@ static void __restore_globals(void)
 int ast_register_application2(const char *app, int (*execute)(struct ast_channel *, const char *),
 				     const char *synopsis, const char *description, void *mod);
 
-/*! 
+/*!
  * \brief Unregister an application
- * 
+ *
  * \param app name of the application (does not have to be the same string as the one that was registered)
- * 
+ *
  * This unregisters an application from Asterisk's internal application list.
- * 
- * \retval 0 success 
+ *
+ * \retval 0 success
  * \retval -1 failure
  */
 int ast_unregister_application(const char *app);
