@@ -173,8 +173,6 @@
 #include "asterisk/utils.h"
 #include "asterisk/event.h"
 
-/*! @{ */
-
 /*!
  * \brief Metadata about a \ref stasis_message.
  * \since 12
@@ -416,14 +414,14 @@ const struct timeval *stasis_message_timestamp(const struct stasis_message *msg)
  * May return \c NULL, to indicate no representation. The returned object should
  * be ast_json_unref()'ed.
  *
- * \param message Message to convert to JSON string.
+ * \param msg Message to convert to JSON string.
  * \param sanitize Snapshot sanitization callback.
  *
  * \return Newly allocated string with JSON message.
  * \return \c NULL on error.
  * \return \c NULL if JSON format is not supported.
  */
-struct ast_json *stasis_message_to_json(struct stasis_message *message, struct stasis_message_sanitizer *sanitize);
+struct ast_json *stasis_message_to_json(struct stasis_message *msg, struct stasis_message_sanitizer *sanitize);
 
 /*!
  * \brief Build the AMI representation of the message.
@@ -431,12 +429,21 @@ struct ast_json *stasis_message_to_json(struct stasis_message *message, struct s
  * May return \c NULL, to indicate no representation. The returned object should
  * be ao2_cleanup()'ed.
  *
- * \param message Message to convert to AMI.
+ * \param msg Message to convert to AMI.
  * \return \c NULL on error.
  * \return \c NULL if AMI format is not supported.
  */
-struct ast_manager_event_blob *stasis_message_to_ami(
-	struct stasis_message *message);
+struct ast_manager_event_blob *stasis_message_to_ami(struct stasis_message *msg);
+
+/*!
+ * \brief Determine if the given message can be converted to AMI.
+ *
+ * \param msg Message to see if can be converted to AMI.
+ *
+ * \retval 0 Cannot be converted
+ * \retval non-zero Can be converted
+ */
+int stasis_message_can_be_ami(struct stasis_message *msg);
 
 /*!
  * \brief Build the \ref AstGenericEvents representation of the message.
@@ -444,16 +451,11 @@ struct ast_manager_event_blob *stasis_message_to_ami(
  * May return \c NULL, to indicate no representation. The returned object should
  * be disposed of via \ref ast_event_destroy.
  *
- * \param message Message to convert to AMI.
+ * \param msg Message to convert to AMI.
  * \return \c NULL on error.
  * \return \c NULL if AMI format is not supported.
  */
-struct ast_event *stasis_message_to_event(
-	struct stasis_message *message);
-
-/*! @} */
-
-/*! @{ */
+struct ast_event *stasis_message_to_event(struct stasis_message *msg);
 
 /*!
  * \brief A topic to which messages may be posted, and subscribers, well, subscribe
@@ -508,10 +510,6 @@ void stasis_publish(struct stasis_topic *topic, struct stasis_message *message);
  */
 void stasis_publish_sync(struct stasis_subscription *sub, struct stasis_message *message);
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \brief Callback function type for Stasis subscriptions.
  * \param data Data field provided with subscription.
@@ -520,6 +518,17 @@ void stasis_publish_sync(struct stasis_subscription *sub, struct stasis_message
  */
 typedef void (*stasis_subscription_cb)(void *data, struct stasis_subscription *sub, struct stasis_message *message);
 
+/*!
+ * \brief Stasis subscription callback function that does nothing.
+ *
+ * \note This callback should be used for events are not directly processed, but need
+ * to be generated so data can be retrieved from cache later.  Subscriptions with this
+ * callback can be released with \ref stasis_unsubscribe, even during module unload.
+ *
+ * \since 13.5
+ */
+void stasis_subscription_cb_noop(void *data, struct stasis_subscription *sub, struct stasis_message *message);
+
 /*!
  * \brief Create a subscription.
  *
@@ -581,6 +590,20 @@ struct stasis_subscription *stasis_subscribe_pool(struct stasis_topic *topic,
 struct stasis_subscription *stasis_unsubscribe(
 	struct stasis_subscription *subscription);
 
+/*!
+ * \brief Set the high and low alert water marks of the stasis subscription.
+ * \since 13.10.0
+ *
+ * \param subscription Pointer to a stasis subscription
+ * \param low_water New queue low water mark. (-1 to set as 90% of high_water)
+ * \param high_water New queue high water mark.
+ *
+ * \retval 0 on success.
+ * \retval -1 on error (water marks not changed).
+ */
+int stasis_subscription_set_congestion_limits(struct stasis_subscription *subscription,
+	long low_water, long high_water);
+
 /*!
  * \brief Block until the last message is processed on a subscription.
  *
@@ -699,8 +722,6 @@ struct stasis_message_type *stasis_subscription_change_type(void);
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \brief Pool for topic aggregation
  */
@@ -723,8 +744,6 @@ struct stasis_topic_pool *stasis_topic_pool_create(struct stasis_topic *pooled_t
  */
 struct stasis_topic *stasis_topic_pool_get_topic(struct stasis_topic_pool *pool, const char *topic_name);
 
-/*! @} */
-
 /*! \addtogroup StasisTopicsAndMessages
  * @{
  */
@@ -757,8 +776,6 @@ struct stasis_message_type *stasis_cache_clear_type(void);
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \brief A message cache, for use with \ref stasis_caching_topic.
  * \since 12
@@ -1090,6 +1107,10 @@ struct ao2_container *stasis_cache_dump_by_eid(struct stasis_cache *cache, struc
  */
 struct ao2_container *stasis_cache_dump_all(struct stasis_cache *cache, struct stasis_message_type *type);
 
+/*! \addtogroup StasisTopicsAndMessages
+ * @{
+ */
+
 /*!
  * \brief Object type code for multi user object snapshots
  */
@@ -1163,8 +1184,6 @@ void ast_multi_object_blob_single_channel_publish(struct ast_channel *chan, stru
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \internal
  * \brief Log a message about invalid attempt to access a type.
@@ -1267,10 +1286,6 @@ void stasis_log_bad_type_access(const char *name);
 		_priv_ ## name = NULL;		\
 	})
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \brief Initialize the Stasis subsystem.
  * \return 0 on success.
@@ -1279,10 +1294,6 @@ void stasis_log_bad_type_access(const char *name);
  */
 int stasis_init(void);
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \internal
  * \brief called by stasis_init() for cache initialization.
@@ -1301,12 +1312,10 @@ int stasis_cache_init(void);
  */
 int stasis_config_init(void);
 
-/*! @} */
-
 /*!
  * \defgroup StasisTopicsAndMessages Stasis topics, and their messages.
  *
- * This group contains the topics, messages and corresponding message types
+ * \brief This group contains the topics, messages and corresponding message types
  * found within Asterisk.
  */
 
