@@ -33,8 +33,6 @@
 
 #if defined(__AST_DEBUG_MALLOC)
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
 #include <stddef.h>
 #include <time.h>
@@ -1457,7 +1455,7 @@ static void mm_atexit_dump(void)
 	/*
 	 * Put the alloced list back into regions[].
 	 *
-	 * We have do do this because we can get called before all other
+	 * We have do this because we can get called before all other
 	 * threads have terminated.
 	 */
 	mm_atexit_hash_restore(&alloced_atexit);
