@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "astobj2_private.h"
@@ -648,11 +646,7 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 
 	switch (options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
-#if defined(__AST_DEBUG_MALLOC)
 		obj_mutex = __ast_calloc(1, sizeof(*obj_mutex) + data_size, file, line, func);
-#else
-		obj_mutex = ast_calloc(1, sizeof(*obj_mutex) + data_size);
-#endif
 		if (obj_mutex == NULL) {
 			return NULL;
 		}
@@ -661,11 +655,7 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 		obj = (struct astobj2 *) &obj_mutex->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_RWLOCK:
-#if defined(__AST_DEBUG_MALLOC)
 		obj_rwlock = __ast_calloc(1, sizeof(*obj_rwlock) + data_size, file, line, func);
-#else
-		obj_rwlock = ast_calloc(1, sizeof(*obj_rwlock) + data_size);
-#endif
 		if (obj_rwlock == NULL) {
 			return NULL;
 		}
@@ -674,11 +664,7 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 		obj = (struct astobj2 *) &obj_rwlock->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
-#if defined(__AST_DEBUG_MALLOC)
 		obj = __ast_calloc(1, sizeof(*obj) + data_size, file, line, func);
-#else
-		obj = ast_calloc(1, sizeof(*obj) + data_size);
-#endif
 		if (obj == NULL) {
 			return NULL;
 		}
@@ -690,11 +676,7 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 			return NULL;
 		}
 
-#if defined(__AST_DEBUG_MALLOC)
 		obj_lockobj = __ast_calloc(1, sizeof(*obj_lockobj) + data_size, file, line, func);
-#else
-		obj_lockobj = ast_calloc(1, sizeof(*obj_lockobj) + data_size);
-#endif
 		if (obj_lockobj == NULL) {
 			ao2_t_ref(lockobj, -1, "release lockobj for failed alloc");
 			return NULL;
@@ -756,12 +738,6 @@ unsigned int ao2_options_get(void *obj)
 	return orig_obj->priv_data.options;
 }
 
-
-void __ao2_global_obj_release(struct ao2_global_obj *holder, const char *tag, const char *file, int line, const char *func, const char *name)
-{
-	__ao2_global_obj_replace_unref(holder, NULL, tag, file, line, func, name);
-}
-
 void *__ao2_global_obj_replace(struct ao2_global_obj *holder, void *obj, const char *tag, const char *file, int line, const char *func, const char *name)
 {
 	void *obj_old;
@@ -890,7 +866,7 @@ int __ao2_weakproxy_set_object(void *weakproxy, void *obj, int flags,
 		ao2_unlock(weakproxy);
 		/* It is possible for obj to be accessed now.  It's allowed
 		 * for weakproxy to already be in a container.  Another thread
-		 * could have been waiting for a lock on weakproxy to retreive
+		 * could have been waiting for a lock on weakproxy to retrieve
 		 * the object.
 		 */
 	}
