@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <math.h>
 
 #include "asterisk/lock.h"
@@ -43,23 +41,9 @@ ASTERISK_REGISTER_FILE()
 #include "asterisk/cli.h"
 #include "asterisk/module.h"
 #include "asterisk/astobj2.h"
-#include "asterisk/data.h"
 
 #include "asterisk/_private.h" /* _init(), _reload() */
 
-#define DATA_EXPORT_TONE_ZONE(MEMBER)					\
-	MEMBER(ast_tone_zone, country, AST_DATA_STRING)			\
-	MEMBER(ast_tone_zone, description, AST_DATA_STRING)		\
-	MEMBER(ast_tone_zone, nrringcadence, AST_DATA_UNSIGNED_INTEGER)
-
-AST_DATA_STRUCTURE(ast_tone_zone, DATA_EXPORT_TONE_ZONE);
-
-#define DATA_EXPORT_TONE_ZONE_SOUND(MEMBER)			\
-	MEMBER(ast_tone_zone_sound, name, AST_DATA_STRING)	\
-	MEMBER(ast_tone_zone_sound, data, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(ast_tone_zone_sound, DATA_EXPORT_TONE_ZONE_SOUND);
-
 /* Globals */
 static const char config[] = "indications.conf";
 
@@ -1126,33 +1110,6 @@ static int ast_tone_zone_cmp(void *obj, void *arg, int flags)
 			CMP_MATCH | CMP_STOP : 0;
 }
 
-int ast_tone_zone_data_add_structure(struct ast_data *tree, struct ast_tone_zone *zone)
-{
-	struct ast_data *data_zone_sound;
-	struct ast_tone_zone_sound *s;
-
-	ast_data_add_structure(ast_tone_zone, tree, zone);
-
-	if (AST_LIST_EMPTY(&zone->tones)) {
-		return 0;
-	}
-
-	data_zone_sound = ast_data_add_node(tree, "tones");
-	if (!data_zone_sound) {
-		return -1;
-	}
-
-	ast_tone_zone_lock(zone);
-
-	AST_LIST_TRAVERSE(&zone->tones, s, entry) {
-		ast_data_add_structure(ast_tone_zone_sound, data_zone_sound, s);
-	}
-
-	ast_tone_zone_unlock(zone);
-
-	return 0;
-}
-
 /*!
  * \internal
  * \brief Clean up resources on Asterisk shutdown
@@ -1194,4 +1151,3 @@ int ast_indications_reload(void)
 {
 	return load_indications(1);
 }
-
