@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h" /* ast_ssl_init() */
 
 #ifdef HAVE_OPENSSL
@@ -39,7 +37,7 @@ ASTERISK_REGISTER_FILE()
 #endif
 
 #if defined(HAVE_OPENSSL) && \
-	!defined(OPENSSL_VERSION_NUMBER) || OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+	(!defined(OPENSSL_VERSION_NUMBER) || OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER))
 
 #include <dlfcn.h>
 
