@@ -20,7 +20,7 @@
  * \file
  *
  * \author Russell Bryant <russell@digium.com>
- * 
+ *
  * \brief curses frontend for selection maintenance
  */
 
@@ -194,46 +194,49 @@ static void display_mem_info(WINDOW *menu, struct member *mem, int start_y, int
 	int start_x = (max_x / 2 - MEMBER_INFO_LEFT_ADJ);
 	int maxlen = (max_x - start_x);
 
-	wmove(menu, end - start_y + 1, start_x);
+	wmove(menu, end - start_y + 1, 0);
 	wclrtoeol(menu);
-	wmove(menu, end - start_y + 2, start_x);
+	wmove(menu, end - start_y + 2, 0);
 	wclrtoeol(menu);
-	wmove(menu, end - start_y + 3, start_x);
+	wmove(menu, end - start_y + 3, 0);
 	wclrtoeol(menu);
-	wmove(menu, end - start_y + 4, start_x);
+	wmove(menu, end - start_y + 4, 0);
 	wclrtoeol(menu);
-	wmove(menu, end - start_y + 5, start_x);
+	wmove(menu, end - start_y + 5, 0);
 	wclrtoeol(menu);
-	wmove(menu, end - start_y + 6, start_x);
+	wmove(menu, end - start_y + 6, 0);
+	wclrtoeol(menu);
+	wmove(menu, end - start_y + 7, 0);
 	wclrtoeol(menu);
 
 	if (mem->displayname) {
-		int name_len = strlen(mem->displayname);
+		char buf[maxlen + 1];
+		char *displayname = ast_strdupa(mem->displayname);
+		char *word;
+		int current_line = 1;
+		int new_line = 1;
 
+		buf[0] = '\0';
 		wmove(menu, end - start_y + 1, start_x);
-		if (name_len >  maxlen) {
-			char *last_space;
-			char *line_1 = strdup(mem->displayname);
 
-			if (line_1) {
-				line_1[maxlen] = '\0';
-				last_space = strrchr(line_1, ' ');
-				if (last_space) {
-					*last_space = '\0';
-				}
-				waddstr(menu, line_1);
-				wmove(menu, end - start_y + 2, start_x);
-				waddstr(menu, &mem->displayname[last_space - line_1]);
-				free(line_1);
-			} else {
-				waddstr(menu, (char *) mem->displayname);
+		while ((word = strsep(&displayname, " "))) {
+			if ((strlen(buf) + strlen(word) + 1) > maxlen) {
+				waddstr(menu, buf);
+				current_line++;
+				wmove(menu, end - start_y + current_line, start_x);
+				buf[0] = '\0';
+				new_line = 1;
 			}
-		} else {
-			waddstr(menu, (char *) mem->displayname);
+			sprintf(buf, "%s%*.*s%s", buf, new_line ? 0 : 1, new_line ? 0 : 1, " ", word);
+			new_line = 0;
+		}
+		if (strlen(buf)) {
+			waddstr(menu, buf);
 		}
 	}
+
 	if (!AST_LIST_EMPTY(&mem->deps)) {
-		wmove(menu, end - start_y + 3, start_x);
+		wmove(menu, end - start_y + 4, start_x);
 		strcpy(buf, "Depends on: ");
 		AST_LIST_TRAVERSE(&mem->deps, dep, list) {
 			strncat(buf, dep->displayname, sizeof(buf) - strlen(buf) - 1);
@@ -244,7 +247,7 @@ static void display_mem_info(WINDOW *menu, struct member *mem, int start_y, int
 		waddstr(menu, buf);
 	}
 	if (!AST_LIST_EMPTY(&mem->uses)) {
-		wmove(menu, end - start_y + 4, start_x);
+		wmove(menu, end - start_y + 5, start_x);
 		strcpy(buf, "Can use: ");
 		AST_LIST_TRAVERSE(&mem->uses, use, list) {
 			strncat(buf, use->displayname, sizeof(buf) - strlen(buf) - 1);
@@ -255,7 +258,7 @@ static void display_mem_info(WINDOW *menu, struct member *mem, int start_y, int
 		waddstr(menu, buf);
 	}
 	if (!AST_LIST_EMPTY(&mem->conflicts)) {
-		wmove(menu, end - start_y + 5, start_x);
+		wmove(menu, end - start_y + 6, start_x);
 		strcpy(buf, "Conflicts with: ");
 		AST_LIST_TRAVERSE(&mem->conflicts, con, list) {
 			strncat(buf, con->displayname, sizeof(buf) - strlen(buf) - 1);
@@ -268,7 +271,7 @@ static void display_mem_info(WINDOW *menu, struct member *mem, int start_y, int
 
 	if (!mem->is_separator) { /* Separators lack support levels */
 		{ /* support level */
-			wmove(menu, end - start_y + 6, start_x);
+			wmove(menu, end - start_y + 7, start_x);
 			snprintf(buf, sizeof(buf), "Support Level: %s", mem->support_level);
 			if (mem->replacement && *mem->replacement) {
 				char buf2[64];
@@ -288,7 +291,7 @@ static void draw_category_menu(WINDOW *menu, struct category *cat, int start, in
 	char buf[64];
 
 	if (!changed) {
-		/* If all we have to do is move the cursor, 
+		/* If all we have to do is move the cursor,
 		 * then don't clear the screen and start over */
 		AST_LIST_TRAVERSE(&cat->members, mem, list) {
 			i++;
@@ -324,7 +327,7 @@ static void draw_category_menu(WINDOW *menu, struct category *cat, int start, in
 			snprintf(buf, sizeof(buf), "[%s] %s", mem->enabled ? "*" : " ", mem->name);
 		}
 		waddstr(menu, buf);
-		
+
 		if (curopt + 1 == i)
 			display_mem_info(menu, mem, start, end);
 
@@ -464,7 +467,7 @@ static int run_category_menu(WINDOW *menu, int cat_num)
 			set_all(cat, 1);
 			changed = 1;
 		default:
-			break;	
+			break;
 		}
 		if (c == 'x' || c == 'X' || c == 'Q' || c == 'q')
 			break;
@@ -530,13 +533,13 @@ int run_menu(void)
 	refresh();
 
 	maxopt = count_categories() - 1;
-	
+
 	/* We have two windows - the title window at the top, and the menu window gets the rest */
 	title = newwin(TITLE_HEIGHT, max_x, 0, 0);
 	menu = newwin(max_y - TITLE_HEIGHT, max_x, TITLE_HEIGHT, 0);
-	draw_title_window(title);	
+	draw_title_window(title);
 	draw_main_menu(menu, curopt);
-	
+
 	while ((c = getch())) {
 		switch (c) {
 		case KEY_UP:
@@ -568,7 +571,7 @@ int run_menu(void)
 			play_space();
 			draw_title_window(title);
 		default:
-			break;	
+			break;
 		}
 		if (c == 'q' || c == 'Q' || c == 27 || c == 3) {
 			if (changes_made) {
@@ -583,7 +586,7 @@ int run_menu(void)
 			}
 		}
 		if (c == 'x' || c == 'X' || c == 's' || c == 'S')
-			break;	
+			break;
 		draw_main_menu(menu, curopt);
 	}
 
@@ -726,7 +729,7 @@ static int repaint_screen(void)
 			wmove(stdscr, cur->oy, cur->ox);
 			waddch(stdscr, ' ');
 			wmove(stdscr, cur->y, cur->x);
-			waddch(stdscr, type2chtype(cur->type));	
+			waddch(stdscr, type2chtype(cur->type));
 			cur->ox = cur->x;
 			cur->oy = cur->y;
 		}
@@ -743,7 +746,7 @@ static int tank_move_left(void)
 {
 	if (tank->x > 0)
 		tank->x--;
-	
+
 	return 0;
 }
 
