@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "resource_bridges.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_bridges.h"
@@ -388,7 +386,6 @@ static int ari_bridges_play_helper(const char **args_media,
 
 	if (ast_asprintf(playback_url, "/playbacks/%s",
 			stasis_app_playback_get_id(playback)) == -1) {
-		playback_url = NULL;
 		ast_ari_response_alloc_failed(response);
 		return -1;
 	}
@@ -965,13 +962,12 @@ void ast_ari_bridges_create_with_id(struct ast_variable *headers,
 
 	if (bridge) {
 		/* update */
-		if (!ast_strlen_zero(args->name)) {
-			if (!strcmp(args->name, bridge->name)) {
-				ast_ari_response_error(
-					response, 500, "Internal Error",
-					"Changing bridge name is not implemented");
-				return;
-			}
+		if (!ast_strlen_zero(args->name)
+			&& strcmp(args->name, bridge->name)) {
+			ast_ari_response_error(
+				response, 500, "Internal Error",
+				"Changing bridge name is not implemented");
+			return;
 		}
 		if (!ast_strlen_zero(args->type)) {
 			ast_ari_response_error(
