@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/bridge.h"
@@ -101,9 +99,13 @@ static int channel_state_invalid(struct stasis_app_control *control,
 		|| snapshot->state == AST_STATE_RINGING) {
 		ast_ari_response_error(response, 412, "Precondition Failed",
 			"Channel in invalid state");
+		ao2_ref(snapshot, -1);
+
 		return -1;
 	}
 
+	ao2_ref(snapshot, -1);
+
 	return 0;
 }
 
@@ -886,6 +888,8 @@ void ast_ari_channels_hangup(struct ast_variable *headers,
 		cause = AST_CAUSE_CONGESTION;
 	} else if (!strcmp(args->reason, "no_answer")) {
 		cause = AST_CAUSE_NOANSWER;
+	} else if(!strcmp(args->reason, "answered_elsewhere")) {
+		cause = AST_CAUSE_ANSWERED_ELSEWHERE;
 	} else {
 		ast_ari_response_error(
 			response, 400, "Invalid Reason",
