@@ -23,9 +23,12 @@
  * \author David M. Lee, II <dlee@digium.com>
  */
 
-#include "asterisk.h"
+/*** MODULEINFO
+	<depend type="module">res_http_websocket</depend>
+	<support_level>core</support_level>
+ ***/
 
-ASTERISK_REGISTER_FILE()
+#include "asterisk.h"
 
 #include "resource_events.h"
 #include "asterisk/astobj2.h"
@@ -105,7 +108,9 @@ static void stasis_app_message_handler(
 		        msg_application);
 	} else if (!session->ws_session) {
 		/* If the websocket is NULL, the message goes to the queue */
-		AST_VECTOR_APPEND(&session->message_queue, message);
+		if (!AST_VECTOR_APPEND(&session->message_queue, message)) {
+			ast_json_ref(message);
+		}
 		ast_log(LOG_WARNING,
 				"Queued '%s' message for Stasis app '%s'; websocket is not ready\n",
 				msg_type,
@@ -433,7 +438,7 @@ static int event_session_alloc(struct ast_tcptls_session_instance *ser,
 		if (register_handler(app, stasis_app_message_handler, session)) {
 			ast_log(LOG_WARNING, "Stasis registration failed for application: '%s'\n", app);
 			return event_session_allocation_error_handler(
-				session, ERROR_TYPE_STASIS_REGISTRATION, ser);			
+				session, ERROR_TYPE_STASIS_REGISTRATION, ser);
 		}
 	}
 
