@@ -18,7 +18,7 @@
 
 /*! \file
  * \brief Calendaring API
- * 
+ *
  * \todo Support responding to a meeting invite
  * \todo Support writing attendees
  */
@@ -27,19 +27,17 @@
  * \addtogroup configuration_file Configuration Files
  */
 
-/*! 
+/*!
  * \page calendar.conf calendar.conf
  * \verbinclude calendar.conf.sample
  */
 
 /*** MODULEINFO
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h"
 #include "asterisk/channel.h"
 #include "asterisk/calendar.h"
@@ -403,22 +401,33 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 {
 	struct ast_calendar *cal;
 	struct ast_variable *v, *last = NULL;
+	int new_calendar = 0;
 
-	if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
-		ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
-		return NULL;
-	}
-
-	if (!(cal->events = ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn))) {
-		ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
+	cal = find_calendar(cat);
+	if (cal && cal->fetch_again_at_reload) {
+		/** Create new calendar, old will be removed during reload */
 		cal = unref_calendar(cal);
-		return NULL;
 	}
+	if (!cal) {
+		new_calendar = 1;
+		if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
+			ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
+			return NULL;
+		}
 
-	if (ast_string_field_init(cal, 32)) {
-		ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
-		cal = unref_calendar(cal);
-		return NULL;
+		if (!(cal->events = ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn))) {
+			ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
+			cal = unref_calendar(cal);
+			return NULL;
+		}
+
+		if (ast_string_field_init(cal, 32)) {
+			ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
+			cal = unref_calendar(cal);
+			return NULL;
+		}
+	} else {
+		cal->pending_deletion = 0;
 	}
 
 	ast_string_field_set(cal, name, cat);
@@ -427,6 +436,7 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 	cal->refresh = 3600;
 	cal->timeframe = 60;
 	cal->notify_waittime = 30000;
+	cal->fetch_again_at_reload = 0;
 
 	for (v = ast_variable_browse(cfg, cat); v; v = v->next) {
 		if (!strcasecmp(v->name, "autoreminder")) {
@@ -448,6 +458,8 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 			ast_string_field_set(cal, notify_appdata, v->value);
 		} else if (!strcasecmp(v->name, "refresh")) {
 			cal->refresh = atoi(v->value);
+		} else if (!strcasecmp(v->name, "fetch_again_at_reload")) {
+			cal->fetch_again_at_reload = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "timeframe")) {
 			cal->timeframe = atoi(v->value);
 		} else if (!strcasecmp(v->name, "setvar")) {
@@ -480,15 +492,17 @@ static struct ast_calendar *build_calendar(struct ast_config *cfg, const char *c
 				cal->name);
 	}
 
-	cal->thread = AST_PTHREADT_NULL;
-	ast_cond_init(&cal->unload, NULL);
-	ao2_link(calendars, cal);
-	if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
-		/* If we start failing to create threads, go ahead and return NULL
-		 * and the tech module will be unregistered
-		 */
-		ao2_unlink(calendars, cal);
-		cal = unref_calendar(cal);
+	if (new_calendar) {
+		cal->thread = AST_PTHREADT_NULL;
+		ast_cond_init(&cal->unload, NULL);
+		ao2_link(calendars, cal);
+		if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
+			/* If we start failing to create threads, go ahead and return NULL
+			 * and the tech module will be unregistered
+			 */
+			ao2_unlink(calendars, cal);
+			cal = unref_calendar(cal);
+		}
 	}
 
 	return cal;
@@ -1759,16 +1773,30 @@ static struct ast_custom_function calendar_event_function = {
 	.read = calendar_event_read,
 };
 
+static int cb_pending_deletion(void *user_data, void *arg, int flags)
+{
+	struct ast_calendar *cal = user_data;
+
+	cal->pending_deletion = 1;
+
+	return CMP_MATCH;
+}
+
+static int cb_rm_pending_deletion(void *user_data, void *arg, int flags)
+{
+	struct ast_calendar *cal = user_data;
+
+	return cal->pending_deletion ? CMP_MATCH : 0;
+}
+
 static int reload(void)
 {
 	struct ast_calendar_tech *iter;
 
 	ast_mutex_lock(&reloadlock);
 
-	/* Delete all of the calendars */
-	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
-
-	/* Load configuration */
+	/* Mark existing calendars for deletion */
+	ao2_callback(calendars, OBJ_NODATA | OBJ_MULTIPLE, cb_pending_deletion, NULL);
 	load_config(1);
 
 	AST_LIST_LOCK(&techs);
@@ -1779,6 +1807,9 @@ static int reload(void)
 	}
 	AST_LIST_UNLOCK(&techs);
 
+	/* Delete calendars that no longer show up in the config */
+	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, cb_rm_pending_deletion, NULL);
+
 	ast_mutex_unlock(&reloadlock);
 
 	return 0;
@@ -1859,8 +1890,8 @@ static int unload_module(void)
  * Module loading including tests for configuration or dependencies.
  * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
  * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
- * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
- * configuration file or other non-critical problem return 
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the
+ * configuration file or other non-critical problem return
  * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
  */
 static int load_module(void)
@@ -1902,7 +1933,7 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Asterisk Calendar integration",
-	.support_level = AST_MODULE_SUPPORT_CORE,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload,
