@@ -21,16 +21,15 @@
  */
 
 /*** MODULEINFO
+	<depend>res_calendar</depend>
 	<depend>neon</depend>
 	<depend>ical</depend>
 	<depend>libxml2</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <libical/ical.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -322,7 +321,7 @@ static struct ast_str *caldav_get_events_between(struct caldav_pvt *pvt, time_t
 	return response;
 }
 
-static time_t icalfloat_to_timet(icaltimetype time) 
+static time_t icalfloat_to_timet(icaltimetype time)
 {
 	struct ast_tm tm = {0,};
 	struct timeval tv;
@@ -750,7 +749,7 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk CalDAV Calendar Integration",
-	.support_level = AST_MODULE_SUPPORT_CORE,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
