@@ -21,15 +21,14 @@
  */
 
 /*** MODULEINFO
+	<depend>res_calendar</depend>
 	<depend>neon</depend>
 	<depend>ical</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <libical/ical.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -161,7 +160,7 @@ static icalcomponent *fetch_icalendar(struct icalendar_pvt *pvt)
 	return comp;
 }
 
-static time_t icalfloat_to_timet(icaltimetype time) 
+static time_t icalfloat_to_timet(icaltimetype time)
 {
 	struct ast_tm tm = {0,};
 	struct timeval tv;
@@ -539,7 +538,7 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk iCalendar .ics file integration",
-	.support_level = AST_MODULE_SUPPORT_CORE,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
