@@ -266,7 +266,7 @@ static void sip_transport_state_destroy(void *obj)
 {
 	struct ast_sip_transport_state *state = obj;
 
-	ast_sip_push_task_wait_servant(NULL, destroy_sip_transport_state, state);
+	ast_sip_push_task_synchronous(NULL, destroy_sip_transport_state, state);
 }
 
 /*! \brief Destructor for ast_sip_transport state information */
@@ -650,9 +650,6 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 	} else if ((transport->type == AST_TRANSPORT_WS) || (transport->type == AST_TRANSPORT_WSS)) {
 		if (transport->cos || transport->tos) {
 			ast_log(LOG_WARNING, "TOS and COS values ignored for websocket transport\n");
-		} else if (!ast_strlen_zero(transport->ca_list_file) || !ast_strlen_zero(transport->ca_list_path) ||
-			!ast_strlen_zero(transport->cert_file) || !ast_strlen_zero(transport->privkey_file)) {
-			ast_log(LOG_WARNING, "TLS certificate values ignored for websocket transport as they are configured in http.conf\n");
 		}
 		res = PJ_SUCCESS;
 	}
@@ -979,22 +976,27 @@ static int tls_method_to_str(const void *obj, const intptr_t *args, char **buf)
 /*! \brief Helper function which turns a cipher name into an identifier */
 static pj_ssl_cipher cipher_name_to_id(const char *name)
 {
-	pj_ssl_cipher ciphers[PJ_SSL_SOCK_MAX_CIPHERS];
+	pj_ssl_cipher ciphers[100];
+	pj_ssl_cipher id = 0;
 	unsigned int cipher_num = PJ_ARRAY_SIZE(ciphers);
-	unsigned int pos;
+	int pos;
+	const char *pos_name;
 
 	if (pj_ssl_cipher_get_availables(ciphers, &cipher_num)) {
 		return 0;
 	}
 
 	for (pos = 0; pos < cipher_num; ++pos) {
-		const char *pos_name = pj_ssl_cipher_name(ciphers[pos]);
-		if (pos_name && !strcmp(pos_name, name)) {
-			return ciphers[pos];
+		pos_name = pj_ssl_cipher_name(ciphers[pos]);
+		if (!pos_name || strcmp(pos_name, name)) {
+			continue;
 		}
+
+		id = ciphers[pos];
+		break;
 	}
 
-	return 0;
+	return id;
 }
 
 /*!
@@ -1069,7 +1071,7 @@ static int transport_tls_cipher_handler(const struct aco_option *opt, struct ast
 static void cipher_to_str(char **buf, const pj_ssl_cipher *ciphers, unsigned int cipher_num)
 {
 	struct ast_str *str;
-	unsigned int idx;
+	int idx;
 
 	str = ast_str_create(128);
 	if (!str) {
@@ -1103,7 +1105,7 @@ static int transport_tls_cipher_to_str(const void *obj, const intptr_t *args, ch
 
 static char *handle_pjsip_list_ciphers(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-	pj_ssl_cipher ciphers[PJ_SSL_SOCK_MAX_CIPHERS];
+	pj_ssl_cipher ciphers[100];
 	unsigned int cipher_num = PJ_ARRAY_SIZE(ciphers);
 	char *buf;
 
@@ -1158,19 +1160,10 @@ static int transport_localnet_handler(const struct aco_option *opt, struct ast_v
 	return error;
 }
 
-static void localnet_to_vl_append(struct ast_variable **head, struct ast_ha *ha)
-{
-	char str[MAX_OBJECT_FIELD];
-	const char *addr = ast_strdupa(ast_sockaddr_stringify_addr(&ha->addr));
-	snprintf(str, MAX_OBJECT_FIELD, "%s%s/%s", ha->sense == AST_SENSE_ALLOW ? "!" : "",
-			 addr, ast_sockaddr_stringify_addr(&ha->netmask));
-
-	ast_variable_list_append(head, ast_variable_new("local_net", str, ""));
-}
-
 static int localnet_to_vl(const void *obj, struct ast_variable **fields)
 {
 	const struct ast_sip_transport *transport = obj;
+	char str[MAX_OBJECT_FIELD];
 	struct ast_variable *head = NULL;
 	struct ast_ha *ha;
 	RAII_VAR(struct ast_sip_transport_state *, state, find_state_by_transport(transport), ao2_cleanup);
@@ -1180,7 +1173,11 @@ static int localnet_to_vl(const void *obj, struct ast_variable **fields)
 	}
 
 	for (ha = state->localnet; ha; ha = ha->next) {
-		localnet_to_vl_append(&head, ha);
+		const char *addr = ast_strdupa(ast_sockaddr_stringify_addr(&ha->addr));
+		snprintf(str, MAX_OBJECT_FIELD, "%s%s/%s", ha->sense == AST_SENSE_ALLOW ? "!" : "",
+			addr, ast_sockaddr_stringify_addr(&ha->netmask));
+
+		ast_variable_list_append(&head, ast_variable_new("local_net", str, ""));
 	}
 
 	if (head) {
@@ -1381,8 +1378,7 @@ static int populate_transport_states(void *obj, void *arg, int flags)
 
 struct ao2_container *ast_sip_get_transport_states(void)
 {
-	struct ao2_container *states = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
-		DEFAULT_STATE_BUCKETS, transport_state_hash, NULL, transport_state_cmp);
+	struct ao2_container *states = ao2_container_alloc(DEFAULT_STATE_BUCKETS, transport_state_hash, transport_state_cmp);
 
 	if (!states) {
 		return NULL;
@@ -1399,8 +1395,7 @@ int ast_sip_initialize_sorcery_transport(void)
 	struct ao2_container *transports = NULL;
 
 	/* Create outbound registration states container. */
-	transport_states = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
-		DEFAULT_STATE_BUCKETS, internal_state_hash, NULL, internal_state_cmp);
+	transport_states = ao2_container_alloc(DEFAULT_STATE_BUCKETS, internal_state_hash, internal_state_cmp);
 	if (!transport_states) {
 		ast_log(LOG_ERROR, "Unable to allocate transport states container\n");
 		return -1;
