@@ -342,8 +342,7 @@ static void update_incoming_connected_line(struct ast_sip_session *session, pjsi
 {
 	struct ast_party_id id;
 
-	if (!session->endpoint->trust_connected_line
-		|| !session->endpoint->id.trust_inbound) {
+	if (!session->endpoint->id.trust_inbound) {
 		return;
 	}
 
@@ -431,7 +430,8 @@ static pjsip_fromto_hdr *create_new_id_hdr(const pj_str_t *hdr_name, pjsip_fromt
 
 	id_hdr = pjsip_from_hdr_create(tdata->pool);
 	id_hdr->type = PJSIP_H_OTHER;
-	id_hdr->sname = id_hdr->name = *hdr_name;
+	pj_strdup(tdata->pool, &id_hdr->name, hdr_name);
+	id_hdr->sname = id_hdr->name;
 
 	id_name_addr = pjsip_uri_clone(tdata->pool, base->uri);
 	id_uri = pjsip_uri_get_uri(id_name_addr->uri);
@@ -543,33 +543,6 @@ static void add_pai_header(const struct ast_sip_session *session, pjsip_tx_data
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *)pai_hdr);
 }
 
-/*!
- * \internal
- * \brief Add party parameter to a Remote-Party-ID header.
- *
- * \param tdata The message where the Remote-Party-ID header is
- * \param hdr The header on which the parameters are being added
- * \param session The session involved
- */
-static void add_party_param(pjsip_tx_data *tdata, pjsip_fromto_hdr *hdr, const struct ast_sip_session *session)
-{
-	static const pj_str_t party_str = { "party", 5 };
-	static const pj_str_t calling_str = { "calling", 7 };
-	static const pj_str_t called_str = { "called", 6 };
-	pjsip_param *party;
-
-	/* The party value can't change throughout the lifetime, so it is set only once */
-	party = pjsip_param_find(&hdr->other_param, &party_str);
-	if (party) {
-		return;
-	}
-
-	party = PJ_POOL_ALLOC_T(tdata->pool, pjsip_param);
-	party->name = party_str;
-	party->value = (session->inv_session->role == PJSIP_ROLE_UAC) ? calling_str : called_str;
-	pj_list_insert_before(&hdr->other_param, party);
-}
-
 /*!
  * \internal
  * \brief Add privacy and screen parameters to a Remote-Party-ID header.
@@ -659,7 +632,6 @@ static void add_rpid_header(const struct ast_sip_session *session, pjsip_tx_data
 			pj_list_erase(old_rpid);
 		} else {
 			ast_sip_modify_id_header(tdata->pool, old_rpid, id);
-			add_party_param(tdata, old_rpid, session);
 			add_privacy_params(tdata, old_rpid, id);
 			return;
 		}
@@ -675,7 +647,6 @@ static void add_rpid_header(const struct ast_sip_session *session, pjsip_tx_data
 	if (!rpid_hdr) {
 		return;
 	}
-	add_party_param(tdata, rpid_hdr, session);
 	add_privacy_params(tdata, rpid_hdr, id);
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *)rpid_hdr);
 }
@@ -750,10 +721,7 @@ static void caller_id_outgoing_response(struct ast_sip_session *session, pjsip_t
 	struct ast_party_id effective_id;
 	struct ast_party_id connected_id;
 
-	if (!session->channel
-		|| (!session->endpoint->send_connected_line
-			&& session->inv_session
-			&& session->inv_session->state >= PJSIP_INV_STATE_EARLY)) {
+	if (!session->channel) {
 		return;
 	}
 
@@ -781,7 +749,6 @@ static int load_module(void)
 {
 	CHECK_PJSIP_SESSION_MODULE_LOADED();
 
-	ast_module_shutdown_ref(ast_module_info->self);
 	ast_sip_session_register_supplement(&caller_id_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -793,8 +760,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Caller ID Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+);
