@@ -47,39 +47,49 @@ static int find_transport_state_in_use(void *obj, void *arg, int flags)
 	struct ast_sip_transport_state *transport_state = obj;
 	pjsip_rx_data *rdata = arg;
 
-	if (transport_state && ((transport_state->transport == rdata->tp_info.transport) ||
-		(transport_state->factory && !pj_strcmp(&transport_state->factory->addr_name.host, &rdata->tp_info.transport->local_name.host) &&
-			transport_state->factory->addr_name.port == rdata->tp_info.transport->local_name.port))) {
-		return CMP_MATCH | CMP_STOP;
+	if (transport_state->transport == rdata->tp_info.transport
+		|| (transport_state->factory
+			&& !pj_strcmp(&transport_state->factory->addr_name.host, &rdata->tp_info.transport->local_name.host)
+			&& transport_state->factory->addr_name.port == rdata->tp_info.transport->local_name.port)) {
+		return CMP_MATCH;
 	}
 
 	return 0;
 }
 
+#define DOMAIN_NAME_LEN 255
+
 static struct ast_sip_endpoint *anonymous_identify(pjsip_rx_data *rdata)
 {
-	char domain_name[64], id[AST_UUID_STR_LEN];
+	char domain_name[DOMAIN_NAME_LEN + 1];
 	struct ast_sip_endpoint *endpoint;
-	RAII_VAR(struct ast_sip_domain_alias *, alias, NULL, ao2_cleanup);
-	RAII_VAR(struct ao2_container *, transport_states, NULL, ao2_cleanup);
-	RAII_VAR(struct ast_sip_transport_state *, transport_state, NULL, ao2_cleanup);
-	RAII_VAR(struct ast_sip_transport *, transport, NULL, ao2_cleanup);
 
 	if (get_endpoint_details(rdata, domain_name, sizeof(domain_name))) {
 		return NULL;
 	}
 
 	if (!ast_sip_get_disable_multi_domain()) {
+		struct ast_sip_domain_alias *alias;
+		struct ao2_container *transport_states;
+		struct ast_sip_transport_state *transport_state = NULL;
+		struct ast_sip_transport *transport = NULL;
+		char id[sizeof("anonymous@") + DOMAIN_NAME_LEN];
+
 		/* Attempt to find the endpoint given the name and domain provided */
 		snprintf(id, sizeof(id), "anonymous@%s", domain_name);
-		if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
+		endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+		if (endpoint) {
 			goto done;
 		}
 
 		/* See if an alias exists for the domain provided */
-		if ((alias = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "domain_alias", domain_name))) {
+		alias = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "domain_alias",
+			domain_name);
+		if (alias) {
 			snprintf(id, sizeof(id), "anonymous@%s", alias->domain);
-			if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
+			ao2_ref(alias, -1);
+			endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+			if (endpoint) {
 				goto done;
 			}
 		}
@@ -90,9 +100,13 @@ static struct ast_sip_endpoint *anonymous_identify(pjsip_rx_data *rdata)
 			&& (transport = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "transport", transport_state->id))
 			&& !ast_strlen_zero(transport->domain)) {
 			snprintf(id, sizeof(id), "anonymous@%s", transport->domain);
-			if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
-				goto done;
-			}
+			endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+		}
+		ao2_cleanup(transport);
+		ao2_cleanup(transport_state);
+		ao2_cleanup(transport_states);
+		if (endpoint) {
+			goto done;
 		}
 	}
 
@@ -124,9 +138,8 @@ static int unload_module(void)
 	return 0;
 }
 
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Anonymous endpoint identifier",
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "PJSIP Anonymous endpoint identifier",
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.load_pri = AST_MODPRI_DEFAULT,
 );
