@@ -65,48 +65,63 @@ static int find_transport_state_in_use(void *obj, void *arg, int flags)
 	struct ast_sip_transport_state *transport_state = obj;
 	pjsip_rx_data *rdata = arg;
 
-	if (transport_state && ((transport_state->transport == rdata->tp_info.transport) ||
-		(transport_state->factory && !pj_strcmp(&transport_state->factory->addr_name.host, &rdata->tp_info.transport->local_name.host) &&
-			transport_state->factory->addr_name.port == rdata->tp_info.transport->local_name.port))) {
-		return CMP_MATCH | CMP_STOP;
+	if (transport_state->transport == rdata->tp_info.transport
+		|| (transport_state->factory
+			&& !pj_strcmp(&transport_state->factory->addr_name.host, &rdata->tp_info.transport->local_name.host)
+			&& transport_state->factory->addr_name.port == rdata->tp_info.transport->local_name.port)) {
+		return CMP_MATCH;
 	}
 
 	return 0;
 }
 
+#define DOMAIN_NAME_LEN 255
+#define USERNAME_LEN    255
+
 static struct ast_sip_endpoint *find_endpoint(pjsip_rx_data *rdata, char *endpoint_name,
 	char *domain_name)
 {
-	char id[AST_UUID_STR_LEN];
 	struct ast_sip_endpoint *endpoint;
-	RAII_VAR(struct ast_sip_domain_alias *, alias, NULL, ao2_cleanup);
-	RAII_VAR(struct ao2_container *, transport_states, NULL, ao2_cleanup);
-	RAII_VAR(struct ast_sip_transport_state *, transport_state, NULL, ao2_cleanup);
-	RAII_VAR(struct ast_sip_transport *, transport, NULL, ao2_cleanup);
 
 	if (!ast_sip_get_disable_multi_domain()) {
+		struct ast_sip_domain_alias *alias;
+		struct ao2_container *transport_states;
+		struct ast_sip_transport_state *transport_state = NULL;
+		struct ast_sip_transport *transport = NULL;
+		char id[DOMAIN_NAME_LEN + USERNAME_LEN + sizeof("@")];
+
 		/* Attempt to find the endpoint given the name and domain provided */
 		snprintf(id, sizeof(id), "%s@%s", endpoint_name, domain_name);
-		if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
+		endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+		if (endpoint) {
 			return endpoint;
 		}
 
 		/* See if an alias exists for the domain provided */
-		if ((alias = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "domain_alias", domain_name))) {
+		alias = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "domain_alias",
+			domain_name);
+		if (alias) {
 			snprintf(id, sizeof(id), "%s@%s", endpoint_name, alias->domain);
-			if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
+			ao2_ref(alias, -1);
+			endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+			if (endpoint) {
 				return endpoint;
 			}
 		}
+
 		/* See if the transport this came in on has a provided domain */
 		if ((transport_states = ast_sip_get_transport_states())
 			&& (transport_state = ao2_callback(transport_states, 0, find_transport_state_in_use, rdata))
 			&& (transport = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "transport", transport_state->id))
 			&& !ast_strlen_zero(transport->domain)) {
-			snprintf(id, sizeof(id), "anonymous@%s", transport->domain);
-			if ((endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id))) {
-				return endpoint;
-			}
+			snprintf(id, sizeof(id), "%s@%s", endpoint_name, transport->domain);
+			endpoint = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "endpoint", id);
+		}
+		ao2_cleanup(transport);
+		ao2_cleanup(transport_state);
+		ao2_cleanup(transport_states);
+		if (endpoint) {
+			return endpoint;
 		}
 	}
 
@@ -116,8 +131,8 @@ static struct ast_sip_endpoint *find_endpoint(pjsip_rx_data *rdata, char *endpoi
 
 static struct ast_sip_endpoint *username_identify(pjsip_rx_data *rdata)
 {
-	char username[64];
-	char domain[64];
+	char username[USERNAME_LEN + 1];
+	char domain[DOMAIN_NAME_LEN + 1];
 	struct ast_sip_endpoint *endpoint;
 
 	if (get_from_header(rdata, username, sizeof(username), domain, sizeof(domain))) {
@@ -149,7 +164,7 @@ static struct ast_sip_endpoint *username_identify(pjsip_rx_data *rdata)
 
 static struct ast_sip_endpoint *auth_username_identify(pjsip_rx_data *rdata)
 {
-	char username[64], realm[64];
+	char username[USERNAME_LEN + 1], realm[DOMAIN_NAME_LEN + 1];
 	struct ast_sip_endpoint *endpoint;
 	pjsip_authorization_hdr *auth_header = NULL;
 
