@@ -37,8 +37,6 @@
 
 #include "asterisk.h"                   /* for NULL, size_t, memcpy, etc */
 
-ASTERISK_REGISTER_FILE()
-
 #include <math.h>                       /* for pow */
 
 #if HAVE_SRTP_VERSION > 1
@@ -200,7 +198,7 @@ static struct ast_srtp *res_srtp_new(void)
 		ast_free(srtp);
 		return NULL;
 	}
-	
+
 	srtp->warned = 1;
 
 	return srtp;
@@ -458,11 +456,12 @@ tryagain:
 		 * whether they use a custom library or an outdated version of libSRTP.
 		 */
 		if (rtcp) {
-			ast_verb(2, "SRTCP unprotect failed because of %s\n", srtp_errstr(res));
+			ast_verb(2, "SRTCP unprotect failed on SSRC %u because of %s\n",
+				ast_rtp_instance_get_ssrc(srtp->rtp), srtp_errstr(res));
 		} else {
 			if ((srtp->warned >= 10) && !((srtp->warned - 10) % 150)) {
-				ast_verb(2, "SRTP unprotect failed because of %s %d\n",
-					srtp_errstr(res), srtp->warned);
+				ast_verb(2, "SRTP unprotect failed on SSRC %u because of %s %d\n",
+					ast_rtp_instance_get_ssrc(srtp->rtp), srtp_errstr(res), srtp->warned);
 				srtp->warned = 11;
 			} else {
 				srtp->warned++;
@@ -483,7 +482,7 @@ static int ast_srtp_protect(struct ast_srtp *srtp, void **buf, int *len, int rtc
 	if ((*len + SRTP_MAX_TRAILER_LEN) > sizeof(srtp->buf)) {
 		return -1;
 	}
-	
+
 	localbuf = rtcp ? srtp->rtcpbuf : srtp->buf;
 
 	memcpy(localbuf, *buf, *len);
