@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/app.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/bridge.h"
@@ -499,7 +497,11 @@ struct stasis_app_playback *stasis_app_control_play_uri(
 
 	    /* safe */
 		strcpy(media_uri, media[i]);
-		AST_VECTOR_APPEND(&playback->medias, media_uri);
+		if (AST_VECTOR_APPEND(&playback->medias, media_uri)) {
+			ao2_ref(playback, -1);
+			ast_free(media_uri);
+			return NULL;
+		}
 	}
 
 	if (skipms == 0) {
