@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/message.h"
 #include "asterisk/endpoints.h"
 #include "asterisk/astobj2.h"
@@ -266,6 +264,7 @@ static struct ast_json *msg_to_json(struct ast_msg *msg)
 
 	json_vars = ast_json_array_create();
 	if (!json_vars) {
+		ast_msg_var_iterator_destroy(it_vars);
 		return NULL;
 	}
 
@@ -274,7 +273,8 @@ static struct ast_json *msg_to_json(struct ast_msg *msg)
 
 		json_tuple = ast_json_pack("{s: s}", name, value);
 		if (!json_tuple) {
-			ast_json_free(json_vars);
+			ast_json_unref(json_vars);
+			ast_msg_var_iterator_destroy(it_vars);
 			return NULL;
 		}
 
@@ -457,7 +457,11 @@ static struct message_subscription *get_or_create_subscription(struct ast_endpoi
 		ao2_link(endpoint_subscriptions, sub);
 	} else {
 		ast_rwlock_wrlock(&tech_subscriptions_lock);
-		AST_VECTOR_APPEND(&tech_subscriptions, ao2_bump(sub));
+		if (AST_VECTOR_APPEND(&tech_subscriptions, ao2_bump(sub))) {
+			/* Release the ao2_bump that was for the vector and allocation references. */
+			ao2_ref(sub, -2);
+			sub = NULL;
+		}
 		ast_rwlock_unlock(&tech_subscriptions_lock);
 	}
 
@@ -485,7 +489,11 @@ int messaging_app_subscribe_endpoint(const char *app_name, struct ast_endpoint *
 		ao2_unlock(sub);
 		return -1;
 	}
-	AST_VECTOR_APPEND(&sub->applications, tuple);
+	if (AST_VECTOR_APPEND(&sub->applications, tuple)) {
+		ao2_ref(tuple, -1);
+		ao2_unlock(sub);
+		return -1;
+	}
 	ao2_unlock(sub);
 
 	ast_debug(3, "App '%s' subscribed to messages from endpoint '%s'\n", app_name, endpoint ? ast_endpoint_get_id(endpoint) : "-- ALL --");
