@@ -18,8 +18,6 @@
 #include <regex.h>
 #include <limits.h>
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/backtrace.h"
 #include "asterisk/channel.h"
 #include "asterisk/ast_expr.h"
@@ -32,10 +30,6 @@ ASTERISK_REGISTER_FILE()
 
 int option_debug = 0;
 int option_verbose = 0;
-#if !defined(LOW_MEMORY)
-void __ast_register_file(const char *file) { }
-void __ast_unregister_file(const char *file) { }
-#endif
 
 /*** MODULEINFO
   	<depend>res_ael_share</depend>
@@ -180,7 +174,6 @@ struct ast_custom_function *ast_custom_function_find(const char *name)
 	return 0; /* in "standalone" mode, functions are just not avail */
 }
 
-#if !defined(LOW_MEMORY)
 int ast_add_profile(const char *x, uint64_t scale)
 {
 	if (!no_comp)
@@ -188,7 +181,6 @@ int ast_add_profile(const char *x, uint64_t scale)
 
 	return 0;
 }
-#endif
 
 int ast_loader_register(int (*updater)(void))
 {
@@ -606,7 +598,6 @@ unsigned int ast_hashtab_hash_contexts(const void *obj)
 }
 
 #ifdef DEBUG_THREADS
-#if !defined(LOW_MEMORY)
 void ast_mark_lock_acquired(void *lock_addr)
 {
 }
@@ -652,5 +643,4 @@ void ast_suspend_lock_info(void *lock_addr)
 void ast_restore_lock_info(void *lock_addr)
 {
 }
-#endif /* !defined(LOW_MEMORY) */
 #endif /* DEBUG_THREADS */
