@@ -41,7 +41,7 @@
 /*!
  * \brief reads next USC character from null terminated UTF-8 string and advanced pointer
  * for non valid UTF-8 sequences.
- * \return character as is Does \b NOT advance pointer for null termination 
+ * \return character as is Does \b NOT advance pointer for null termination
 */
 static int utf8decode (unsigned char **pp)
 {
@@ -92,12 +92,12 @@ static int utf8decode (unsigned char **pp)
    return *p;                   /* not sensible */
 }
 
-/*! 
+/*!
  * \brief check for any queued messages in specific queue (queue="" means any queue)
  * \param dir,queue,subaddress,channel,callerid,wait,delay,retries,concurrent
  * \retval 0 if nothing queued
  * \retval 1 if queued and outgoing set up OK
- * \retval 2 of outgoing exists 
+ * \retval 2 of outgoing exists
 */
 static char txqcheck (char *dir, char *queue, char subaddress, char *channel, char *callerid, int wait, int delay, int retries, int concurrent)
 {
@@ -193,7 +193,7 @@ static char txqcheck (char *dir, char *queue, char subaddress, char *channel, ch
    return 2;
 }
 
-/*! 
+/*!
  * \brief Process received queue entries
  * Run through a process, setting environment variables
 */
@@ -390,14 +390,14 @@ static void rxqcheck (char *dir, char *queue, char *process)
             {
                for (n = 0, x = 0; x < udl; x++)
                {
-                  sprintf (tmp + n, "%02X", ud[x]);
+                  sprintf (tmp + n, "%02hX", ud[x]);
                   n += 2;
                }
                setenv ("ud8", tmp, 1);
             }
             for (n = 0, x = 0; x < udl; x++)
             {
-               sprintf (tmp + n, "%04X", ud[x]);
+               sprintf (tmp + n, "%04hX", ud[x]);
                n += 4;
             }
             setenv ("ud16", tmp, 1);
@@ -730,13 +730,13 @@ main (int argc, const char *argv[])
             {                   /* use one byte hex */
                fprintf (f, "ud#");
                for (p = 0; p < udl; p++)
-                  fprintf (f, "%02X", ud[p]);
+                  fprintf (f, "%02hX", ud[p]);
             }
          } else
          {                      /* use two byte hex */
             fprintf (f, "ud##");
             for (p = 0; p < udl; p++)
-               fprintf (f, "%04X", ud[p]);
+               fprintf (f, "%04hX", ud[p]);
          }
          fprintf (f, "\n");
       }
